@@ -109,6 +109,10 @@ Defaults({"log_level": "WARNING"})
 Defaults({"len_date": 8})
 Defaults({"len_time": 6})
 
+# Define whether to call compress_user(), when printing the attributes
+# of a Data object, for each string corresponding to a file path.
+Defaults({"compress_user": False})
+
 
 class Data:
     """
@@ -191,7 +195,7 @@ class Data:
                             try:
                                 item_string = item.__repr__(depth=(depth - 1))
                             except TypeError:
-                                item_string = item.__repr__()
+                                item_string = self.get_item_string(item)
                             comma = "," if (i + 1 < n) else ""
                             value_string = f"{value_string} {item_string}{comma}"
                         value_string = f"{value_string}]"
@@ -212,7 +216,7 @@ class Data:
                             try:
                                 item_string += item.__repr__(depth=(depth - 1))
                             except TypeError:
-                                item_string += item.__repr__()
+                                item_string += self.get_item_string(item)
                             comma = "," if (i + 1 < n) else ""
                             value_string = f"{value_string} {item_string}{comma}"
                         value_string = f"{{{value_string}}}"
@@ -233,12 +237,30 @@ class Data:
                     else:
                         value_string = f"{item.__class__}"
                 else:
-                    value_string = item.__repr__()
+                    value_string = self.get_item_string(item)
             out.append(f"  {key}: {value_string} ")
 
         out.append("}")
         return "\n".join(out)
 
+    def get_item_string(self, item):
+        """
+        Return string representation of an item.
+
+        If Defaults().compress_user is set to True, the function
+        compress_user() is called for each item corresponding to a file
+        path, where the file or its parent directory exists.
+        """
+        if (
+                Defaults().compress_user
+                and isinstance(item, (str, Path))
+                and (os.path.exists(str(item))
+                     or os.path.exists(os.path.dirname(str(item))))
+                ):
+            return compress_user(item)
+        else:
+            return item.__repr__()
+
     def clone(self, **kwargs):
         """
         Return a clone of the Data object. All attributes of the original
@@ -724,6 +746,33 @@ class PathData(Data):
         # Only 1 data file associated with a non-Archive object.
         return 1
 
+    def get_n_file_below(self):
+        """
+        Where self.path is a directory, return number of files below this.
+
+        Returns None if self.path isn't the path to a directory.  In
+        counting files, hidden files are ignored.
+        """
+        return get_n_file_below(self.path)
+
+    def print_paths(self, max_path=None):
+        '''
+        Print paths of data files associated with this object.
+
+        File paths are listed in natural order, with one path per line.
+        
+        **Parameters:**
+        max_path: int/None, default=None
+            Indication of maximum number of paths to print.  If a positive
+            integer, the first <max_path> paths are printed.  If a negative
+            integer, the last <max_path> paths are printed.  If None,
+            all paths are printed.
+        '''
+        if os.path.isdir(self.path):
+            print_paths(self.path, max_path)
+        else:
+            print(self.path)
+
 
 class Dated(PathData):
     """PathData with an associated date and time, which can be used for
@@ -831,7 +880,7 @@ class Dated(PathData):
         '''
         return (self > other) or (self == other)
 
-    def copy_dicom(self, outdir=None, overwrite=True, sort=True):
+    def copy_dicom(self, outdir=None, overwrite=True, sort=True, index=None):
         """
         Copy (single) source dicom file.
 
@@ -856,20 +905,29 @@ class Dated(PathData):
             If True, the copied dicom file will be given name of form
             'MODALITY_YYMMDD_hhmmss'.  If False, the file is copied
             with name unaltered.
+
+        index : int, default=None
+            Integer representing associated object position in an ordered
+            list of objects of source data type.  Used in construction
+            of output filename if not None, otherwise Ignored.
         """
         if not callable(getattr(self, "load", None)):
             raise NotImplementedError(
                     f"{type(self)}.copy_dicom() failed - "
                     "class has no load() method")
             return
-        self.load()
+
+        # Create clone for data loading.
+        # Clone is deleted once data are no longer needed.
+        obj = self.clone()
+        obj.load()
 
         # Check that object has dicom file to be copied.
-        path = Path(self.path)
-        ds = getattr(self, "dicom_dataset", None)
+        path = Path(obj.path)
+        ds = getattr(obj, "dicom_dataset", None)
         if not ds or not path.exists():
             raise NotImplementedError(
-                    f"{type(self)}.copy_dicom() failed - "
+                    f"{type(obj)}.copy_dicom() failed - "
                     "object has no associated DICOM file")
             return
 
@@ -881,7 +939,10 @@ class Dated(PathData):
 
         # Define path to the output file.
         if sort:
-            name = f"{modality}_{self.timestamp}.dcm" if sort else path.name()
+            idx = "" if index is None else f"_{index+1:03}"
+            name = f"{modality}_{obj.timestamp}{idx}.dcm"
+        else:
+            name = path.name()
         outpath = outdir / name
 
         # Copy file.
@@ -925,13 +986,13 @@ class Dated(PathData):
         if (isinstance(indices, dict)
                 and isinstance(data_type, str)
                 and isinstance(indices.get(data_type, None), list)
-                and isinstance(data_index, int)
-                and data_index not in indices[data_type]):
+                and isinstance(index, int)
+                and index not in indices[data_type]):
             return
 
         # Create output directory, and copy object files.
         outdir = make_dir(outdir, overwrite)
-        self.copy_dicom(outdir, overwrite, sort)
+        self.copy_dicom(outdir, overwrite, sort, index)
 
 class MachineData(Dated):
     """Dated object with an associated machine name."""
@@ -2141,3 +2202,53 @@ def qualified__name(cls):
     """
     if isinstance(cls, type):
         return f"{cls.__module__}.{cls.__name__}"
+
+def get_n_file_below(indir):
+    """
+    Return number of files below a directory, ignoring hidden files.
+
+    Returns None if indir isn't the path to a directory.
+
+    **Parameter:**
+    indir: str, pathlib.Path
+        Path to directory below which files are to be counted.
+    """
+    if isinstance(indir, (str, Path)) and str(indir):
+        indir = Path(indir)
+        if indir.is_dir():
+            return len([path for path in indir.glob("**/[!.]*")
+                        if path.is_file()])
+
+def print_paths(data_dir, max_path=None):
+    """
+    Print paths to files below a directory, ignoring hidden files.
+    
+    File paths are listed in natural order, with one path per line.
+    
+    **Parameters:**
+    data_dir: str, pathlib.Path
+        Path to directory below which file paths are to be printed.
+        
+    max_path: int/None, default=None
+        Indication of maximum number of paths to print.  If a positive
+        integer, the first <max_path> paths are printed.  If a negative
+        integer, the last <max_path> paths are printed.  If None,
+        all paths are printed.
+    """
+    # Obtain sorted list of paths.
+    local_paths = sorted(
+            list(Path(data_dir).glob("**/[!.]*")), key=alphanumeric)
+
+    # Reduce number of paths as needed.
+    if max_path is None:
+        selected_paths = local_paths
+    else:
+        if max_path >= 0:
+            selected_paths = local_paths[: max_path]
+        else:
+            selected_paths = local_paths[max_path:]
+
+    # Print paths.
+    for path in selected_paths:
+        out_path = compress_user(path) if Defaults().compress_user else path
+        print(out_path)
