@@ -36,6 +36,10 @@ class ImageOverlay(skrt.image.Image):
             image = skrt.image.Image(image)
         self.image = image
 
+    def get_image(self):
+        """Return associated image."""
+        return self.image
+
     def plot(
         self, 
         view=None,
@@ -70,8 +74,9 @@ class ImageOverlay(skrt.image.Image):
         **Parameters**:
 
         view : str
-            Orientation in which to compute the index. Can be "x-y", "y-z", or
-            "x-z".  If None, the initial view is chosen to match
+            Orientation in which to compute the index. Can be "x-y", "x-z",
+            "y-x", "y-z", "z-x", or
+            "z-y".  If None, the initial view is chosen to match
             the image orienation.
 
         sl : int, default=None
@@ -230,8 +235,15 @@ class ImageOverlay(skrt.image.Image):
 
 
 class Dose(ImageOverlay):
-    """Class representing a dose map. The same as an Image but with overridden
-    plotting behaviour and extra functionality relating to ROIs."""
+    """
+    Class representing a radiotherapy dose field.  This is the
+    same as the Image class, but with overridden
+    plotting behaviour, and with extra functionality relating to ROIs.
+
+    Attributes of a Dose object should usually be accessed via
+    their getter methods, rather than directly, to ensure that
+    attribute values have been loaded.
+    """
 
     def __init__(self, *args, **kwargs):
 
@@ -254,7 +266,8 @@ class Dose(ImageOverlay):
         self.dose_units = None
         self.dose_type = None
         self.dose_summation_type = None
-        
+        self.n_fraction = None
+        self._equivalent_dose = None
 
     def load(self, *args, **kwargs):
         """Load self and set default maximum plotting intensity from max of
@@ -313,6 +326,12 @@ class Dose(ImageOverlay):
         self.load()
         return self.dose_summation_type
 
+    def get_n_fraction(self):
+        """Return number of dose fractions."""
+        if self.n_fraction is None and hasattr(self, "plan"):
+            self.n_fraction = self.plan.get_n_fraction()
+        return self.n_fraction
+
     def get_dose_in_roi_3d(self, roi, standardise=True):
         """Return copy of the dose array that has values retained
         inside an ROI, and set to zero elsewhere.  Fails if the
@@ -446,6 +465,10 @@ class Dose(ImageOverlay):
 
         return ax
 
+    def plot_DVH(self, *args, **kwargs):
+        """Alias for plot_dvh()."""
+        self.plot_dvh(*args, **kwargs)
+
     def get_mean_dose(self, roi):
         """Get mean dose inside an ROI."""
 
@@ -468,9 +491,11 @@ class Dose(ImageOverlay):
 
         See aliased method for documentation.
         """
+        self._equivalent_dose = "get_bed()"
         return self.get_biologically_effective_dose(*args, **kwargs)
 
-    def get_biologically_effective_dose(self, rois=None, alpha_beta_ratios=None,
+    def get_biologically_effective_dose(
+            self, rois=None, alpha_beta_ratios=None,
             n_fraction=None, fill=0, standardise=False, force=False):
         """
         Get a Dose object where array values are biologically effective dose.
@@ -484,11 +509,11 @@ class Dose(ImageOverlay):
         **Parameters:**
 
         rois : ROI/StructureSet/list, default=None
-            Object(s) specifying ROIs for which biologically effective
-            dose is to be determined.  The object(s) can be a
-            single skrt.structures.ROI object, a single
-            skrt.structures.StructureSet object, or a list containing any
-            combination of ROI and StructureSet objects.
+            Object(s) specifying ROIs for which calculation is to be
+            performed.  The object(s) can be a single
+            skrt.structures.ROI object, a single skrt.structures.StructureSet
+            object, or a list containing any combination of ROI
+            and StructureSet objects.
 
         alpha_beta_ratios : dict, default=None
             By default, values of alpha over beta used are the values
@@ -499,15 +524,15 @@ class Dose(ImageOverlay):
             for alpha over beta.
 
         n_fraction : float, default=None
-            Number of equal-dose fractions over which physical dose
-            is delivered.  If None, and the dose has an associated plan,
-            the value returned by self.plan.get_n_fraction() is used.
+            Number of equal-dose fractions over which physical total dose
+            is delivered.  If None, the value returned by
+            self.get_n_fraction() is used.
 
-        fill : float/str, default=0
+        fill : float/None, default=0
             Specification of dose value to set outside of ROIs, and for ROIs
             where alpha over beta has a null value.  If a float, this is
-            used as the default dose value.  If 'physical_dose", physical-dose
-            values are retained.
+            used as the default dose value.  If None, physical-dose values
+            are retained.
 
         standardise : bool, default=False
             If False, the data arrays for Doses and for ROI masks will be
@@ -518,80 +543,183 @@ class Dose(ImageOverlay):
         force : bool, default=False
             Determine action if the current dose object has
             dose_type set to "EFFECTIVE".  If False, write an error message,
-            and don't apply the formula for biologically effective dose.  If
+            and don't apply the formula for recalculating dose.  If
             True, apply the formula, even if the dose_type suggests that
-            values already represent biologically effective dose.
+            dose values have already been corrected for biological effect.
         """
-        # Check that the input parameter allow
-        # calculation of biologically effective dose.
+        self._equivalent_dose = "get_biologically_effective_dose()"
+        return self.get_equivalent_dose(
+                dose_rate=0, rois=rois, alpha_beta_ratios=alpha_beta_ratios,
+                n_fraction=n_fraction, fill=fill, standardise=standardise,
+                force=force)
 
-        if not isinstance(fill, numbers.Number) and fill != "physical_dose":
+    def get_eqd(self, *args, **kwargs):
+        """
+        Alias for skrt.dose.Dose method get_equivalent_dose().
+
+        See aliased method for documentation.
+        """
+        self._equivalent_dose = "get_eqd()"
+        return self.get_equivalent_dose(*args, **kwargs)
+
+    def get_equivalent_dose(
+            self, dose_rate=2, rois=None, alpha_beta_ratios=None,
+            n_fraction=None, fill=0, standardise=False, force=False):
+        """
+        Get a Dose object where array values are equivalent dose
+        for a specified dose per fraction.
+
+        If a physical dose, D, is delivered over n equal fractions to a
+        volume element characterised by linear and quadratic coefficients
+        alpha and beta for the linear-quadratic equation, the total equivalent
+        dose, EQDd, at a dose rate d per fraction, is:
+            EQDd = D * ((D/n) + (alpha/beta)) / (d + (alpha/beta)).
+        It approaches the biologically effective dose, BED,
+        as d tends to zero:
+            EQD0 = D * (1 + (D/n) / (alpha/beta)) = BED.
+
+        **Parameters:**
+
+        dose_rate : float, default=2
+            Dose (Gy) per fraction for which calculation is to be determined.
+
+        rois : ROI/StructureSet/list, default=None
+            Object(s) specifying ROIs for which calculation is to be
+            performed.  The object(s) can be a single
+            skrt.structures.ROI object, a single skrt.structures.StructureSet
+            object, or a list containing any combination of ROI
+            and StructureSet objects.
+
+        alpha_beta_ratios : dict, default=None
+            By default, values of alpha over beta used are the values
+            set for individual ROIs, for example using
+            skrt.StructureSet.set_alpha_beta_ratios().  The default
+            values can be overridden here by passing a dictionary
+            where keys are ROI names and values are revised values
+            for alpha over beta.
+
+        n_fraction : float, default=None
+            Number of equal-dose fractions over which physical total dose
+            is delivered.  If None, the value returned by
+            self.get_n_fraction() is used.
+
+        fill : float/None, default=0
+            Specification of dose value to set outside of ROIs, and for ROIs
+            where alpha over beta has a null value.  If a float, this is
+            used as the default dose value.  If None, physical-dose values
+            are retained.
+
+        standardise : bool, default=False
+            If False, the data arrays for Doses and for ROI masks will be
+            kept in the orientation in which they were loaded; otherwise,
+            they will be converted to standard dicom-style orientation,
+            such that [column, row, slice] corresponds to the [x, y, z] axes.
+
+        force : bool, default=False
+            Determine action if the current dose object has
+            dose_type set to "EFFECTIVE".  If False, write an error message,
+            and don't apply the formula for recalculating dose.  If
+            True, apply the formula, even if the dose_type suggests that
+            dose values have already been corrected for biological effect.
+        """
+        # Ensure that a method name is set.
+        if self._equivalent_dose is None:
+            self._equivalent_dose = "get_equivalent_dose()"
+
+        # Check input values.
+        if not isinstance(fill, numbers.Number) and fill is not None:
             self.logger.error(f"Value of fill set to '{fill}' "
-                    "but must be 'physical_dose' or a number.")
+                    "but must be a number or None.")
+            self._equivalent_dose = None
             return
 
         self.load()
         if "EFFECTIVE" == self.dose_type and not force:
-            self.logger.error("Method get_biologically_effective_dose() called "
+            self.logger.error(f"Method {self._equivalent_dose} called "
                     f"for dose with type '{self.dose_type}'.  "
                     "If this is the intention, repeat call with 'force=True'")
+            self._equivalent_dose = None
             return
 
         rois = skrt.structures.get_all_rois(rois)
         if not rois:
-            self.logger.error("Method get_biologically_effective_dose() called "
+            self.logger.error(f"Method {self._equivalent_dose} called "
                 "without specifying any ROIs.")
+            self._equivalent_dose = None
             return
 
         if not n_fraction:
-            if hasattr(self, "plan"):
-                n_fraction = self.plan.get_n_fraction()
+            n_fraction = self.get_n_fraction()
 
         if not n_fraction:
-            self.logger.error("Method get_biologically_effective_dose() called "
+            self.logger.error(f"Method {self._equivalent_dose} called "
                 "without specifying number of fractions.")
+            self._equivalent_dose = None
             return
 
         # Set default dose to be fill value.
         if isinstance(fill, numbers.Number):
-            bed = Dose(
+            dose = Dose(
                     path=(fill *
                         np.ones(self.get_data(standardise=standardise).shape)),
                     affine=self.get_affine(standardise=standardise))
-        # Set default dose to be physical dose.
-        elif "physical_dose" == fill:
-            bed = Dose(
+        # Set default dose to be initial dose.
+        elif fill is None:
+            dose = Dose(
                     path=self.get_data(standardise=standardise),
                     affine=self.get_affine(standardise=standardise))
 
-        # Calculate biologically effective dose
-        # inside ROIs with alpha/beta defined.
+        # Calculate equivalent dose inside ROIs with alpha/beta defined.
         for roi in rois:
             # For current ROI, use alpha/beta given in input,
             # or use the value assigned to the ROI.
             alpha_over_beta = alpha_beta_ratios.get(
                     roi.name, roi.alpha_over_beta)
             # If alpha/beta is defined,
-            # calculate the biologically effective dose.
+            # calculate the equivalent dose.
             if alpha_over_beta is not None:
                 dose_in_roi = self.get_dose_in_roi_3d(
                         roi, standardise=standardise)
-                dose_in_roi += ((dose_in_roi * dose_in_roi)
-                        / (n_fraction * alpha_over_beta))
-                bed.data[dose_in_roi > 0] = dose_in_roi[dose_in_roi > 0]
+                dose_in_roi *= ((alpha_over_beta + dose_in_roi / n_fraction)
+                                / (dose_rate + alpha_over_beta))
+                dose.data[dose_in_roi > 0] = dose_in_roi[dose_in_roi > 0]
 
         # Reset the default maximum plotting intensity
         # from the maximum of the data array.
-        bed._default_vmax = bed.data.max()
+        dose._default_vmax = dose.data.max()
 
-        # Set dose_type to indicate that this is biologically effective dose.
-        bed.dose_type = "EFFECTIVE"
+        # Set dose_type to indicate that biological effect
+        # is taken into account.
+        dose.dose_type = "EFFECTIVE"
 
-        return bed
+        # Unset method name - it will be reset by the next caller.
+        self._equivalent_dose = None
+        return dose
 
 class Plan(skrt.core.Archive):
-    def __init__(self, path="", load=True):
+    """
+    Class representing a radiotherapy plan.
 
+    Attributes of a Plan object should usually be accessed via
+    their getter methods, rather than directly, to ensure that
+    attribute values have been loaded.
+    """
+
+    def __init__(self, path="", load=True):
+        """
+        Instantiate Plan from a DICOM rtplan file.
+
+        **Parameters:**
+
+        path: str/pathlib.Path, default=""
+            Path to DICOM rtplan file from which plan is to be instantiated.
+
+        load: bool, default=True
+            If True, load plan data immediately.  If False, plan data will
+            be loaded automatically the first time that a Plan method
+            requiring the data is called, or may be explicitly loaded
+            by calling the load() method.
+        """
         self.loaded = False
         self.path = None
         self.name = None
@@ -618,10 +746,15 @@ class Plan(skrt.core.Archive):
             self.load()
 
     def load(self, force=False):
-        '''
+        """
         Load plan data.
-        '''
 
+        **Parameter:**
+
+        force: bool, default=False
+            If True, plan data will be reloaded from file, even if
+            loaded previously.
+        """
         if self.loaded and not force:
             return
 
@@ -675,10 +808,14 @@ class Plan(skrt.core.Archive):
         self.load_constraints()
 
     def load_constraints(self, force=False):
-        '''
+        """
         Load dose constraints from plan.
-        '''
 
+        **Parameter:**
+        force: bool, default=False
+            If True, dose constraints will be reloaded from file, even if
+            loaded previously.
+        """
         if ((self.constraints_loaded and not force) or not self.structure_set):
             return
 
@@ -723,30 +860,29 @@ class Plan(skrt.core.Archive):
         self.constraints_loaded = True
 
     def get_dicom_dataset(self):
-        '''
+        """
         Return pydicom.dataset.FileDataset object associated with this plan.
-        '''
-
+        """
         self.load()
         return self.dicom_dataset
 
     def get_targets(self):
-        '''
+        """
         Return list of ROIs identified in plan as targets.
-        '''
+        """
         self.load()
         return self.targets
 
     def get_organs_at_risk(self):
-        '''
+        """
         Return list of ROIs identified in plan as organs at risk.
-        '''
+        """
         self.load()
         return self.organs_at_risk
 
     def get_dose_objective(self, objective='maximum_dose', idx_dose=0,
             dose=None):
-        '''
+        """
         Obtain Dose object representing weight or objective of dose constraint.
 
         **Parameters:**
@@ -761,15 +897,13 @@ class Plan(skrt.core.Archive):
         dose : skrt.dose.Dose, default=None
             Dose object from which dose data are to be taken.  If specified,
             idx_dose is ignored.
-        '''
-
+        """
         # Check that specified objective is known.
         if objective not in Constraint.get_weight_and_objectives():
             print(f'Unknown dose objective: \'{objective}\'')
             print(f'Known objectives: {Constraint.get_weight_and_objectives()}')
             return None
 
-
         # Return pre-existing result if available.
         dose_objective = getattr(self.objectives, objective, None)
         if dose_objective:
@@ -807,42 +941,42 @@ class Plan(skrt.core.Archive):
         return dose_objective
 
     def get_approval_status(self):
-        '''Return plan approval status.'''
+        """Return plan approval status."""
         self.load()
         return self.approval_status
 
     def get_description(self):
-        '''Return plan description.'''
+        """Return plan description."""
         self.load()
         return self.description
 
     def get_prescription_description(self):
-        '''Return plan prescription description.'''
+        """Return plan prescription description."""
         self.load()
         return self.prescription_description
 
     def get_n_beam_seq(self):
-        '''Return number of beam sequences for this plan.'''
+        """Return number of beam sequences for this plan."""
         self.load()
         return self.n_beam_seq
 
     def get_n_fraction(self):
-        '''Return number of fractions for this plan.'''
+        """Return number of fractions for this plan."""
         self.load()
         return self.n_fraction
 
     def get_n_fraction_group(self):
-        '''Return number of fraction groups for this plan.'''
+        """Return number of fraction groups for this plan."""
         self.load()
         return self.n_fraction_group
 
     def get_name(self):
-        '''Return plan name.'''
+        """Return plan name."""
         self.load()
         return self.name
 
     def get_target_dose(self):
-        '''Return dose to target (tumour) for this plan.'''
+        """Return dose to target (tumour) for this plan."""
         self.load()
         return self.target_dose
 
@@ -860,6 +994,10 @@ class Plan(skrt.core.Archive):
         if self.image is not None:
             self.image.add_plan(self)
 
+    def get_image(self):
+        """Return Image object associated with this plan."""
+        return self.image
+
     def set_structure_set(self, structure_set):
         """Set associated structure set, initialising it if needed."""
 
@@ -875,6 +1013,10 @@ class Plan(skrt.core.Archive):
         if self.structure_set is not None:
             self.structure_set.add_plan(self)
 
+    def get_structure_set(self):
+        """Return StructureSet object associated with this plan."""
+        return self.structure_set
+
     def add_dose(self, dose):
         """Add a Dose object to be associated with this plan. This does not
         affect the plan associated with the Dose object.
@@ -893,6 +1035,9 @@ class Plan(skrt.core.Archive):
 
         self.doses = []
 
+    def get_doses(self):
+        """Return Dose objects associated with this plan."""
+        return self.doses
 
 class Constraint(skrt.core.Data):
     '''
