@@ -51,6 +51,7 @@ from copy import deepcopy
 from inspect import signature
 from itertools import combinations
 from pathlib import Path
+import random
 from shutil import rmtree
 
 import pandas as pd
@@ -58,7 +59,10 @@ import pandas as pd
 from skrt.core import (
         Data, Defaults, get_dict_permutations, get_logger, get_stat_functions,
         is_list, tic, toc)
-from skrt.image import Image, match_images
+from skrt.image import (
+        Image, get_mi_metrics, get_quality_metrics, match_images,
+        match_images_for_comparison)
+from skrt.patient import Patient
 from skrt.structures import (
         StructureSet,
         get_by_slice_methods, get_comparison_metrics, get_consensus_types)
@@ -68,6 +72,7 @@ class MultiAtlasSegmentation(Data):
     def __init__(
         self, im1=None, im2=None, ss1=None, ss2=None, log_level=None,
         workdir="segmentation_workdir", overwrite=False,
+        engine=None, engine_dir=None,
         auto=False, auto_step=None, auto_strategies=None,
         auto_reg_setup_only=False,
         default_step=None, default_strategy="pull", roi_names=None,
@@ -96,16 +101,20 @@ class MultiAtlasSegmentation(Data):
         if self.overwrite and self.workdir.exists():
             rmtree(self.workdir)
 
+        # Set registration engine and software directory.
+        self.engine = engine
+        self.engine_dir = engine_dir
+
         # Define contour-propagation strategies and segmentation steps.
-        args = [kwargs.get(key, None) for key in ["engine", "engine_dir"]]
-        self.strategies = get_contour_propagation_strategies(*args)
+        self.strategies = get_contour_propagation_strategies(
+                "engine", "engine_dir")
         self.steps = get_segmentation_steps()
 
         # Note that these values are passed to the SingleAtlasSegmentation
         # constructor, and automatic segmenation should always be enabled.
         self.auto = True
 
-        # Set step and strategies for automatic segmenation,
+        # Set step and strategies for automatic segmentation,
         # default step and strategy, and names or ROIs to be segmented.
         self.auto_step = get_option(auto_step, default_step, self.steps)
         self.default_step = get_option(default_step, self.auto_step, self.steps)
@@ -165,6 +174,7 @@ class MultiAtlasSegmentation(Data):
                 active_ids.append(idx)
                 args = (self.im1, self.im2[idx], self.ss1, self.ss2[idx],
                         self.log_level, self.workdir / str(idx), self.overwrite,
+                        self.engine, self.engine_dir,
                         sas_auto, self.auto_step, self.auto_strategies,
                         reg_setup_only, self.default_step,
                         self.default_strategy, self.roi_names,)
@@ -301,9 +311,6 @@ class MultiAtlasSegmentation(Data):
             name_as_index=False, atlas_ids_to_compare=False,
             combination_length=None, **kwargs):
 
-        if not hasattr(self, "ss1_filtered"):
-            return
-
         consensus_types = get_options(
                 consensus_types, self.consensus_types, get_consensus_types())
         if not (is_list(atlas_ids) and atlas_ids and is_list(atlas_ids[0])):
@@ -331,6 +338,9 @@ class MultiAtlasSegmentation(Data):
 
         sas_id = get_options(atlas_ids_to_compare, True, all_atlas_ids)[0]
         sas = self.get_sas(sas_id)
+        if getattr(sas, "ss1_filtered", None) is None:
+            return
+
         step_reg_steps = {}
         for step in steps:
             all_reg_steps = sas.get_reg_steps(step)
@@ -522,15 +532,16 @@ class SingleAtlasSegmentation(Data):
     def __init__(
         self, im1=None, im2=None, ss1=None, ss2=None, log_level=None,
         workdir="segmentation_workdir", overwrite=False,
-        engine=None, engine_dir=None,
-        auto=False, auto_step=None, auto_strategies=None,
-        auto_reg_setup_only=False,
+        engine=None, engine_dir=None, auto=False, auto_step=None,
+        auto_strategies=None, auto_reg_setup_only=False,
         default_step=-1, default_strategy="pull", roi_names=None,
         ss1_index=-1, ss2_index=-1, ss1_name="Filtered1", ss2_name="Filtered2",
         initial_crop_focus=None, initial_crop_margins=None,
-        initial_alignment=None, initial_transform_name=None,
-        crop_to_match_size1=True, voxel_size1=None, bands1=None, pfiles1=None, 
+        initial_crop_about_centre=False, initial_alignment=None,
+        initial_transform_name=None, crop_to_match_size1=True,
+        voxel_size1=None, bands1=None, pfiles1=None, 
         most_points1=True, roi_crop_margins=None, default_roi_crop_margins=10,
+        roi_crop_about_centre=None, default_roi_crop_about_centre=False,
         roi_pfiles=None, crop_to_match_size2=True, voxel_size2=None,
         default_roi_bands=None, roi_bands=None, pfiles2=None,
         most_points2=True, capture_output=False,
@@ -580,6 +591,7 @@ class SingleAtlasSegmentation(Data):
         self.initial_alignment = initial_alignment
         self.initial_crop_focus = initial_crop_focus
         self.initial_crop_margins = initial_crop_margins
+        self.initial_crop_about_centre = initial_crop_about_centre
         self.crop_to_match_size1 = crop_to_match_size1
         self.voxel_size1 = voxel_size1
         self.bands1 = bands1
@@ -590,6 +602,8 @@ class SingleAtlasSegmentation(Data):
         # Set parameters for step-2 registration.
         self.roi_crop_margins = roi_crop_margins or {}
         self.default_roi_crop_margins = default_roi_crop_margins
+        self.roi_crop_about_centre = roi_crop_about_centre or {}
+        self.default_roi_crop_about_centre = default_roi_crop_about_centre
         self.crop_to_match_size2 = crop_to_match_size2
         self.voxel_size2 = voxel_size2
         self.roi_bands = roi_bands or {}
@@ -653,13 +667,21 @@ class SingleAtlasSegmentation(Data):
                         roi_names=self.roi_names,
                         im2_crop_focus=self.initial_crop_focus,
                         im2_crop_margins=self.initial_crop_margins,
+                        im2_crop_about_centre=self.initial_crop_about_centre,
                         alignment=(self.initial_alignment
                                    if self.crop_to_match_size1 else False),
                         voxel_size=self.voxel_size1,
                         bands=self.bands1)
 
-                self.ss1_filtered = im1.structure_sets[0]
+                if im1.structure_sets:
+                    self.ss1_filtered = im1.structure_sets[0]
+                else:
+                    self.ss1_filtered = None
+
                 self.ss2_filtered = im2.structure_sets[0]
+                if not self.roi_names:
+                    self.roi_names = {roi_name : roi_name for roi_name in
+                                      self.ss2_filtered.get_roi_names()}
 
                 fixed, moving = get_fixed_and_moving(im1, im2, strategy)
 
@@ -706,8 +728,14 @@ class SingleAtlasSegmentation(Data):
                             ss2_name=self.ss2_name,
                             roi_names={roi_name: self.roi_names[roi_name]},
                             im2_crop_focus=roi_name,
-                            im2_crop_margins=self.roi_crop_margins.get(
-                                roi_name, self.default_roi_crop_margins),
+                            im2_crop_margins=(
+                                self.roi_crop_margins.get(
+                                    roi_name,
+                                    self.default_roi_crop_margins)),
+                            im2_crop_about_centre=(
+                                self.roi_crop_about_centre.get(
+                                    roi_name,
+                                    self.default_roi_crop_about_centre)),
                             alignment=(roi_name if self.crop_to_match_size2
                                        else False),
                             voxel_size=self.voxel_size2,
@@ -808,11 +836,6 @@ class SingleAtlasSegmentation(Data):
         else:
             return self.registrations[strategy][step]
 
-    def get_mutual_information(self, strategy=None, step=None, roi_name=None,
-            reg_step=None, force=False, **kwargs):
-        reg = self.get_registration(strategy, step, roi_name, force)
-        return reg.get_mutual_information(reg_step, **kwargs)
-
     def get_segmentation(self, strategy=None, step=None, reg_step=None,
             force=False):
         strategy = get_strategy(strategy, self.default_strategy,
@@ -872,9 +895,9 @@ class SingleAtlasSegmentation(Data):
             steps=None, reg_steps=None, force=False, metrics=None,
             slice_stats=None, default_by_slice=None, 
             voxel_size=None, name_as_index=False,
-            mi_variants=None, mi_init=False, mi_reg_step=False, **kwargs):
+            **kwargs):
 
-        if not hasattr(self, "ss1_filtered"):
+        if getattr(self, "ss1_filtered", None) is None:
             return
 
         strategies = get_options(
@@ -886,24 +909,6 @@ class SingleAtlasSegmentation(Data):
         default_by_slice = get_option(default_by_slice, self.default_by_slice,
                                       get_by_slice_methods())
 
-        if mi_variants is None:
-            mi_variants = ["mi"]
-        elif isinstance(mi_variants, str):
-            mi_variants = [mi_variants]
-        elif not mi_variants:
-            mi_variants = []
-
-        mi_parameters = list(signature(Image.get_mutual_information).parameters)
-        comparison_parameters = list(signature(
-            SingleAtlasSegmentation.get_comparison).parameters)
-        mi_kwargs = {}
-        comparison_kwargs = {}
-        for key, value in kwargs.items():
-            if key in mi_parameters:
-                mi_kwargs[key] = value
-            else:
-                comparison_kwargs[key] = value
-
         df = None
         ss1 = self.ss1_filtered.filtered_copy(to_keep=to_keep)
         ss1.set_image(self.im1, add_to_image=False)
@@ -923,7 +928,7 @@ class SingleAtlasSegmentation(Data):
                             ss2, metrics=metrics, slice_stats=slice_stats,
                             default_by_slice=default_by_slice,
                             voxel_size=voxel_size, name_as_index=name_as_index,
-                            **comparison_kwargs)
+                            **kwargs)
 
                     if df_tmp is None:
                         continue
@@ -935,23 +940,6 @@ class SingleAtlasSegmentation(Data):
                             df_tmp[label] = pd.Series(
                                     df_tmp.shape[0] * [value])
 
-                    if mi_init:
-                        for mi_variant in mi_variants:
-                            mi_kwargs["variant"] = mi_variant
-                            mi = self.get_mutual_information(
-                                    strategy, 0, None, 0, **mi_kwargs)
-                            df_tmp[f"{mi_variant}_init"] = pd.Series(
-                                    df_tmp.shape[0] * [mi])
-
-                    if mi_reg_step:
-                        for mi_variant in mi_variants:
-                            mi_kwargs["variant"] = mi_variant
-                            #print(df_tmp.apply(lambda row: print(row), axis=1))
-                            df_tmp[f"{mi_variant}_reg_step"] = df_tmp.apply(
-                                    lambda row: self.get_mutual_information(
-                                        strategy, step, row["ROI"], reg_step,
-                                        **mi_kwargs), axis=1)
-
                     if df is None:
                         df = df_tmp
                     else:
@@ -1013,6 +1001,256 @@ def ensure_structure_set(ss):
     """Return StructureSet cloned from <im>; return None if <im> is None."""
     return (ss if ss is None else StructureSet(ss))
 
+def get_atlases(paths, subtypes=None, subdirs=None, roi_names=None,
+                structure_set_index=None, unsorted_dicom=True, max_atlas=None):
+    """
+    Obtain dictionary of atlas tuples (structure set and associated image).
+
+    By default, all structure sets in each dataset are considered, in
+    reverse order of timestamp, and the first structure set with an
+    associated image is accepted.  It's possible to consider only
+    a specified structure set, or only structure sets that include specified
+    ROIs.
+
+    In the dictionary returned, keys are identifiers, and values are
+    tuples of the form (skrt.image.Image, skrt.structures.StructureSet).
+
+    **Parameters:**
+
+    paths: str/pathlib.Path/list
+        Path, or list of paths, to patient datasets to be considered as atlases.
+
+    subtypes: 
+        String, or list of strings, identifying subtype(s) of
+        imaging modality, for example "ct", "mr", "us", to be considered
+        as atlases.  If None, all imaging modalities are considered.
+
+    subdirs : str/list, default=None
+        Subdirectory, or list of subdirectories, grouping studies.
+        If specified, only studies in this subdirectory, or
+        in these subdirectories, are considered.  Disregarded if
+        unsorted_dicom is True.
+
+    structure_set_index: int, default=None
+        Index, across all studies, of the structure set to be considered.
+        If None, each structure set is considered, in reverse order
+        of timestamp.  Setting the index to be different from None can
+        be useful if, for example, only the earliest structure set
+        (index 0), or only the most-recent structure set (index -1),
+        should be considered.
+
+    unsorted_dicom: bool, default=True
+        If False, assume that data in a patient dataset are organised
+        according to the VoxTox model.  If True, don't assume any particular
+        organisation, and create data hierarchy based on information
+        read from DICOM files.
+
+    max_atlas: int, default=None
+        Maximum number of atlases to be returned.  If None, there is
+        no maximum.
+    """
+    logger = get_logger(identifier="funcName")
+
+    # If single path given as input, convert to a single-element list.
+    if isinstance(paths, (str, Path)):
+        paths = [paths]
+
+    # Ensure value set for maximum number of atlases to be returned.
+    if max_atlas is None:
+        max_atlas = len(paths)
+
+    # Disallow study subdirectories for unsorted data.
+    if unsorted_dicom:
+        subdirs = None
+
+    # Create dictionary of atlas tuples.
+    atlases = {}
+    for path in paths:
+        if len(atlases) >= max_atlas:
+            break
+
+        # Read dataset.
+        logger.info(f"Reading atlas dataset from: '{path}'")
+        atlas = Patient(path, unsorted_dicom=unsorted_dicom)
+
+        # Load structure sets.
+        structure_sets = atlas.get_structure_sets(subtypes, subdirs)
+
+        # Consider only structure set identified by a particular index.
+        if structure_set_index is not None:
+            try:
+                structure_sets = [structure_sets[structure_set_index]]
+            except IndexError:
+                structure_sets = []
+
+        # Intialise atlas image and structure set.
+        im_atlas = None
+        ss_atlas = None
+
+        # Consider structure sets in order of timestamp,
+        # starting from most recent.
+        for structure_set in reversed(structure_sets):
+            # Disregard structure set if it doesn't have an associated image.
+            im_atlas = structure_set.get_image()
+            if im_atlas is None:
+                continue
+
+            # Disregard structure set if it doesn't contain specified ROIs.
+            if roi_names:
+                ss_atlas = structure_set.filtered_copy(
+                        names=roi_names, keep_renamed_only=True,
+                        copy_roi_data=False)
+                if len(ss_atlas.get_roi_names()) != len(roi_names):
+                    ss_atlas = None
+            else:
+                ss_atlas = structure_set
+
+            if ss_atlas is not None:
+                break
+
+        # Add entry to atlas dictionary.
+        if im_atlas is not None and ss_atlas is not None:
+            atlases[atlas.id] = (im_atlas, ss_atlas)
+
+    return atlases
+
+def select_atlases(atlases, target=None, n_atlas=None, exclude_ids=None,
+                   selection=None, high_to_low=True, alignment=None,
+                   **kwargs):
+    """
+    Select atlases to register against target.
+
+    **Parameters:**
+    atlases: dict
+        Dictionary of atlas tuples (structure set and associated image),
+        such as returned by skrt.segmentation.get_atlases().
+
+    target: skrt.image.Image, default=None
+        Image against which to register.  When selection ordering
+        isn't based on a comparison metric, this can be None.
+
+    n_atlas: int, default=None
+        Number of atlases to select, from those given as input.  If None,
+        all input atlases are returned, ordered according to selection.
+
+    exclude_ids: list, default=None
+        List of atlas identifiers to be excluded from selection.  This may
+        be used, for example, if the target image is also an atlas,
+        and self-registration isn't wanted.  If None, no atlases are
+        excluded.
+
+    selection: str/None, default=None
+        Ordering to be performed prior to selecting the first
+        <n_atlas> atlases:
+
+        - None: the input order is retained;
+
+        - "random": atlases are ordered randomly;
+
+        - "sorted": atlases are sorted in alphabetic order of identifier;
+
+        - "reverse_sorted": atlases are sorted in reverse alphabetic order
+          of identifier;
+
+        - image-comparison metric: atlases are compared with the target,
+          and are ordered according to the specified image-comparison metric.
+          Possibilities include "mutual-information",
+          "information quality ratio", "fidelity"  For a list of available
+          metrics, see documentation for skrt.image.Image.get_comparison().
+
+        - foreground-comparison metric: atlas foregrounds are compared with
+          the target foreground and are ordered according to the specified
+          foreground-comparison metric.  Possibilities include "dice",
+          "centroid", "volume_ratio".  For a list of available metrics,
+          see documentation for skrt.structures.ROI.get_comparison().
+          
+    high_to_low: bool, default=True
+        When ordering for selection is based on a comparison metric,
+        the ordering is from high to low if high_to_low is True, or
+        otherwise is from low to high.
+
+    alignment: tuple/dict/str, default=None
+        Strategy to be used for aligning atlases and target when
+        comparing them.  For strategy details, see documentation of
+        skrt.image.get_alignment_translation().  The value set is
+        passed to skrt.image.match_images_for_comparison(), to match
+        atlas and target prior to evaluation of a comparison metric.
+        Disregarded when ordering for selection isn't based on
+        a comparison metric.
+
+    kwargs: dict
+        Keyword arguments passed to comparison methods:
+        skrt.image.get_comparison() or skrt.image.get_foreground_comparison(),
+        depending on selection specified.
+    """
+    # If no constraints defined, return input atlases.
+    if n_atlas is None and exclude_ids is None and selection is None:
+        return atlases
+
+    logger = get_logger(identifier="funcName")
+
+    # Check that specified selection is allowed.
+    metric_independent_selections = [None, "random", "sorted", "reverse_sorted"]
+    allowed_selections = (metric_independent_selections + get_mi_metrics()
+                          + get_quality_metrics() + get_comparison_metrics())
+    if selection not in allowed_selections:
+        raise RuntimeError(f"Selection {selection} not allowed; "
+                           f"allowed selections: {allowed_selections}")
+
+    # Create list of non-excluded identifiers.
+    if exclude_ids:
+        if not is_list(exclude_ids):
+            exclude_ids = [exclude_ids]
+        atlas_ids = [atlas_id for atlas_id in atlases
+                     if atlas_id not in exclude_ids]
+    else:
+        atlas_ids = list(atlases)
+
+    # Check number of atlases against request.
+    n_atlas = n_atlas if n_atlas is not None else len(atlas_ids)
+    if n_atlas > len(atlas_ids):
+        raise RuntimeError(f"Requested selection of {n_atlas} atlases, "
+                           f"but only {len(atlas_ids)} atlases "
+                           f"after exclusions: {atlas_ids}")
+
+    # Deal with selections not requiring metric evaluations.
+    if selection in metric_independent_selections:
+        if "random" == selection:
+            atlas_ids = random.sample(atlas_ids, n_atlas)
+        elif "sorted" == selection:
+            atlas_ids.sort()
+        elif "reverse_sorted" == selection:
+            atlas_ids.sort(reverse=True)
+        return {atlas_id: atlases[atlas_id] for atlas_id in atlas_ids}
+
+    # Determine type of comparison to be performed for selection.
+    if selection in get_image_comparison_metrics():
+        comparison_method = "get_comparison"
+    else:
+        comparison_method = "get_foreground_comparison"
+    kwargs["metrics"] = [selection]
+
+    # Obtain comparison scores.
+    scores = {}
+    for atlas_id in list(atlas_ids):
+        im1, im2 = match_images_for_comparison(
+            target, atlases[atlas_id][0], alignment=alignment)
+        df = getattr(im1, comparison_method)(im2, **kwargs)
+        score = df.iloc[0][selection]
+        # Save list of atlas identifiers for each score.
+        if score not in scores:
+            scores[score] = []
+        scores[score].append(atlas_id)
+
+    # Sort scores, from high to low or from low to high,
+    # then loop through to select the requested number of atlases.
+    selected_atlases = {}
+    for score, atlas_ids in sorted(scores.items(), reverse=high_to_low):
+        for atlas_id in atlas_ids:
+            selected_atlases[atlas_id] = atlases[atlas_id]
+            if len(selected_atlases) >= n_atlas:
+                return selected_atlases
+
 def get_contour_propagation_strategies(engine=None, engine_dir=None):
     """
     Return list of contour propagation strategies for registration engine.
@@ -1040,13 +1278,8 @@ def get_contour_propagation_strategies(engine=None, engine_dir=None):
         engine = Defaults().registration_engine
 
     # Return engine strategies for contour propagation.
-    if engine in engines:
-        if engines[engine].transform_points_implemented == True:
-            return ["pull", "push"]
-        else:
-            return ["pull"]
-
-    return []
+    return (engines[engine].get_transform_strategies() if engine in engines
+            else [])
 
 def get_fixed_and_moving(im1, im2, strategy):
     """
