@@ -7,17 +7,20 @@ import os
 import random
 import pytest
 import numpy as np
+import pandas as pd
 import shutil
 import pydicom
 
 from pydicom._storage_sopclass_uids import\
         PositronEmissionTomographyImageStorage
 
-from skrt.core import File, fullpath
-from skrt.image import (Image, get_alignment_translation, get_geometry,
-                        get_mask_bbox, get_translation_to_align, match_images,
-                        match_image_voxel_sizes, checked_crop_limits)
+from skrt.core import Defaults, File, fullpath
+from skrt.image import (Image, checked_crop_limits, get_alignment_translation,
+                        get_geometry, get_image_comparison_metrics,
+                        get_mask_bbox, get_translation_to_align,
+                        match_images, match_image_voxel_sizes, rescale_images)
 from skrt.simulation import SyntheticImage
+from skrt.structures import ROI
 
 try:
     import mahotas
@@ -822,13 +825,33 @@ def test_crop_by_amounts():
         assert im1.get_extents()[i_ax][1] - dv2 == im2.get_extents()[i_ax][1]
 
 def test_crop_to_roi():
+    """Text cropping to ROI and StructureSet."""
+    # Create test image, featuring cuboid.
     sim = SyntheticImage((10, 12, 10), origin=(0.5, 0.5, 0.5), noise_std=100)
     sim.add_cuboid((4, 2, 6), name="cuboid")
+    ss = sim.get_structure_set()
     roi = sim.get_roi("cuboid")
-    im = sim.get_image()
-    im.crop_to_roi(roi)
-    for i in range(2):
-        assert set(roi.get_extents()[i]) == set(im.image_extent[i])
+
+    for obj in [roi, ss]:
+        # Check that image extents, after cropping to an ROI or StructureSet,
+        # are the same as the extents of the ROI or StructureSet.
+        im = sim.get_image().clone()
+        im.crop_to_roi(obj)
+        assert np.all(obj.get_extents()
+                      == [list(extent) for extent in im.get_extents()])
+
+        # Check that image extents, after cropping about the centre
+        # of an ROI or StructureSet, are as expected from the
+        # centre coordinates of the ROI or StructureSet plus margins.
+        im = sim.get_image().clone()
+        crop_margins = (2, 1, (-3, 3))
+        im.crop_to_roi(obj, crop_margins=crop_margins, crop_about_centre=True)
+        centre = obj.get_centre()
+        margins = checked_crop_limits(crop_margins)
+        assert np.all(im.get_centre() == centre)
+        extents = [(centre[idx] + margins[idx][0],
+                    centre[idx] + margins[idx][1]) for idx in range(3)]
+        assert np.all(extents == im.get_extents())
 
 def test_crop_to_image():
     # Create test images.
@@ -865,12 +888,11 @@ def test_create_foreground_mask():
     for intensity in [50, 60]:
         threshold = intensity - 5
         mask1 = sim.get_foreground_mask(threshold=threshold).get_data()
-        mask1 = mask1.astype(np.uint32)
-        mask1[mask1 > 0] =1
+        assert mask1.dtype == bool
 
         nx, ny, nz = sim.get_n_voxels()
         mask2 = np.zeros((ny, nx, nz), dtype=np.uint32)
-        mask2[sim.get_data() == intensity] = 1
+        mask2[sim.get_data() == intensity] = True
 
         assert mask1.shape == mask2.shape
         assert mask1.min() == 0
@@ -880,6 +902,41 @@ def test_create_foreground_mask():
         assert mask1.sum() == mask2.sum()
     assert np.all(mask1 == mask2)
 
+@needs_mahotas
+def test_get_foreground_roi():
+    """Test creation of ROI representing image foreground."""
+    # Create synthetic image, featuring cube and sphere.
+    sim = SyntheticImage((100, 100, 40))
+    cube_length = 5
+    sphere_radius = 10
+
+    shapes = {
+            "cube": [cube_length, (25, 60, 12), 60, cube_length**3],
+            "sphere": [sphere_radius, (80, 20, 12), 50,
+                       (4. / 3.) * math.pi * sphere_radius**3],
+            }
+
+    for name, values in shapes.items():
+        length, centre, intensity, volume = values
+        if "cube" == name:
+            sim.add_cube(length, centre, intensity, name=name)
+        elif "sphere" == name:
+            sim.add_sphere(length, centre, intensity, name=name)
+
+    # Should detect sphere as foreground in first loop and cube in second.
+    for name, values in shapes.items():
+        length, centre, intensity, volume = values
+        threshold = intensity - 5
+        roi = sim.get_foreground_roi(threshold=threshold, name=name)
+        assert isinstance(roi, ROI)
+        assert roi.name == name
+        assert roi.get_volume() == pytest.approx(volume, rel=0.005)
+        assert tuple(roi.get_centre()) == centre
+
+    # For case where no name is specified, check that default is assigned.
+    roi = sim.get_foreground_roi(threshold=threshold)
+    assert roi.name == Defaults().foreground_name
+
 @needs_mahotas
 def test_mask_bbox():
     """Test calculation of mask bounding box."""
@@ -941,26 +998,38 @@ def test_foreground_bbox_centre_and_widths():
 
 @needs_mahotas
 def test_create_intensity_mask():
+    """
+    Test creation of intensity masks.
+    """
+    # Create synthetic image (default intensity -1024),
+    # featuring cube and sphere of higher intensities.
     sim = SyntheticImage((100, 100, 40))
-    sim.add_cube(side_length=5, name="cube", centre=(25, 60, 12), intensity=60)
-    sim.add_sphere(radius=10, name="sphere", centre=(80, 20, 12), intensity=50)
+    sim.add_cube(
+            side_length=5, name="cube", centre=(25, 60, 12), intensity=60.4)
+    sim.add_sphere(
+            radius=10, name="sphere", centre=(80, 20, 12), intensity=50.6)
 
+    # Create expected masks given different minimum and maximum intensities.
     nx, ny, nz = sim.get_n_voxels()
-    ones = np.ones((ny, nx, nz), dtype=np.uint32)
+    ones = np.ones((ny, nx, nz), dtype=bool)
     masks = [
             ((None, None), ones),
-            ((45, 55), ones * (sim.get_data() == 50)),
-            ((55, None), ones * (sim.get_data() == 60)),
-            ((None, 55),  1 - ones * (sim.get_data() == 60)),
+            ((50.59, 50.61), ones * (sim.get_data() == 50.6)),
+            ((60, None), ones * (sim.get_data() == 60.4)),
+            ((None, 55),  1 - ones * (sim.get_data() == 60.4)),
             ]
 
+    # Check intensitiy masks against expected masks.
     for intensities, mask2 in masks:
         vmin, vmax = intensities
         mask1 = sim.get_intensity_mask(vmin, vmax).get_data()
-        mask1 = mask1.astype(np.uint32)
         mask1[mask1 > 0] = 1
 
         assert mask1.shape == mask2.shape
+        # If there are no constraints on vmin and vmax,
+        # all elements of the intensity mask should be True
+        # (minimum and maximum both 1).
+        # Otherwise, the minimum should be 0 and the maximum should be 1.
         if vmin is None and vmax is None:
             assert mask1.min() == 1
         else:
@@ -969,6 +1038,9 @@ def test_create_intensity_mask():
         assert mask1.min() == mask2.min()
         assert mask1.max() == mask2.max()
         assert mask1.sum() == mask2.sum()
+
+        # Check that intensity mask is voxel for voxel identical
+        # to the created mask.
         assert np.all(mask1 == mask2)
 
 @needs_mahotas
@@ -976,10 +1048,14 @@ def test_translation_to_align():
     """Test calculation of translation to align pair of images."""
 
     # Create synthetic images featuring a sphere.
-    shapes = [(100, 100, 40), (80, 80, 50)]
-    origins = [(-50, 40, -20), (0, -10, 30)]
-    centres = [(30, 80, 0), (60, 40, 50)]
-    radii = (10, 15)
+    # shapes = [(100, 100, 40), (80, 80, 50)]
+    # origins = [(-50, 40, -20), (0, -10, 30)]
+    # centres = [(30, 80, 0), (60, 40, 50)]
+    # radii = (10, 15)
+    shapes = [(50, 50, 20), (40, 40, 25)]
+    origins = [(-25, 20, -10), (0, -5, 15)]
+    centres = [(15, 40, 0), (30, 20, 25)]
+    radii = (5, 8)
     intensity=50
     sims = []
     for idx in range(len(shapes)):
@@ -1477,3 +1553,130 @@ def test_get_mutual_information():
                     math.log(4, (base or math.e)), small_number)
             assert (im1.get_mutual_information(im2, base=base, variant=variant)
                     == test_value)
+
+def test_rescale_images():
+    """Test rescaling of image greyscale values."""
+    
+    # Create test image, and obtain greyscale characteristics.
+    im = create_test_image(shape, voxel_size, origin)
+    im.data = im.data - (0.5 * im.get_max())
+    u_min = im.get_min(force=True)
+    u_max = im.get_max(force=True)
+    du = u_max - u_min
+    u_sum = im.data.sum()
+
+    # Rescale image.
+    v_min = 0.
+    v_max = 100.
+    constant = 50.
+    dv = v_max - v_min
+    im2 = rescale_images(im, v_min, v_max, constant, clone=False)[0]
+
+    # Check greyscale characteristics of rescaled image.
+    assert im2 is im
+    assert im2.get_min(force=True) == v_min
+    assert im2.get_max(force=True) == v_max
+    assert ((u_min + ((im2.data - v_min) * (du / dv))).sum()
+            == pytest.approx(u_sum, rel=0.001))
+
+    # Check greyscale value after rescaling,
+    # when initial greyscale values are all the same.
+    v_fill = 10
+    im.data.fill(v_fill)
+    im2 = rescale_images((im,), v_min, v_max, constant, clone=True)[0]
+    assert im2 is not im
+    assert np.all(im2.data == constant)
+    assert np.all(im2.data != im.data)
+
+    # Check that original image is returned
+    # when lower or upper bound for rescaling is None,
+    for v_min2, v_max2 in [(None, v_max), (v_min, None)]:
+        im2 = rescale_images([im], v_min2, v_max2, constant, clone=True)[0]
+        assert im2 is im
+        assert np.all(im2.data == v_fill)
+
+def test_get_relative_structural_content():
+    """Test calculation of relative structural content."""
+    # For image compared with itself,
+    # check that relative structural content is 1.
+    im1 = create_test_image(shape, voxel_size, origin)
+    assert im1.get_relative_structural_content(im1) == 1
+
+    # For image of zeros, compared with another image,
+    # check that relative structural content is 0.
+    im2 = create_test_image(shape, voxel_size, origin, "zeros")
+    assert im2.get_relative_structural_content(im1) == 0
+
+def test_get_fidelity():
+    """Test calculation of fidelity."""
+    # For image compared with itself, check that fidelity is 1.
+    im1 = create_test_image(shape, voxel_size, origin)
+    assert im1.get_fidelity(im1) == 1
+
+    # For image of zeros, compared with another image,
+    # check that fidelity is 0.
+    im2 = create_test_image(shape, voxel_size, origin, "zeros")
+    assert im2.get_fidelity(im1) == 0
+
+def test_get_correlation_quality():
+    """Test calculation of correlation quality."""
+    # For image compared with itself, check that correlation_quality is 1.
+    im1 = create_test_image(shape, voxel_size, origin)
+    assert im1.get_correlation_quality(im1) == 1
+
+    # For image of zeros, compared with another image,
+    # check that correlation quality is 0.
+    im2 = create_test_image(shape, voxel_size, origin, "zeros")
+    assert im2.get_correlation_quality(im1) == 0
+
+def test_get_quality():
+    """Test quality of image with respect to another image."""
+    # Create test images, and perform comparison.
+    im1 = create_test_image(shape, voxel_size, origin)
+    im2 = create_test_image(shape, voxel_size, origin)
+
+    metrics = {
+            "relative_structural_content" : 1,
+            "fidelity" : 0.5,
+            "correlation_quality" : 0.75
+            }
+
+    # Check that only None values returned for null or invalid metrics.
+    for metric in [[], "", "unknown_metric"]:
+        scores = im1.get_quality(im2, metric)
+        assert isinstance(scores, dict)
+        assert len(scores) == len(metrics)
+        assert all([score is None for score in scores.values()])
+
+    # Check that scores are as expected for images with random intensity values.
+    scores = im1.get_quality(im2, list(metrics))
+    assert isinstance(scores, dict)
+    assert len(scores) == len(metrics)
+    for metric, score in metrics.items():
+        assert scores[metric] == pytest.approx(score, abs=0.02)
+
+def test_get_image_comparison_metrics():
+    """Check that get_image_comparison_metrics() returns a list of strings."""
+    metrics = get_image_comparison_metrics()
+    assert isinstance(metrics, list)
+    for metric in metrics:
+        assert isinstance(metric, str)
+
+def test_get_comparison():
+    """Test evaluation of image-comparison metrics."""
+    # Create test images, and perform comparison.
+    im1 = create_test_image(shape, voxel_size, origin)
+    im2 = create_test_image(shape, voxel_size, origin)
+    metrics = get_image_comparison_metrics()
+    comparison = im1.get_comparison(im2, metrics=metrics)
+
+    # Check that resulting DataFrame is as expected.
+    assert isinstance(comparison, pd.DataFrame)
+    assert comparison.shape[0] == 1
+    assert comparison.shape[1] == len(metrics)
+    assert list(comparison.columns) == metrics
+
+    # Check that exception is raised for unknown metric.
+    with pytest.raises(RuntimeError) as error_info:
+        comparison = im1.get_comparison(im2, metrics=["unknown_metric"])
+    assert "Metric unknown_metric not recognised" in str(error_info.value)
