@@ -1,4 +1,5 @@
 """Test the skrt.registration.Registration class."""
+from inspect import signature
 import numpy as np
 import os
 from pathlib import Path
@@ -6,13 +7,16 @@ import shutil
 import pytest
 import subprocess
 
+import matplotlib
+
 from skrt import Image
-from skrt.core import Defaults
+from skrt.core import Defaults, fullpath
 from skrt.simulation import SyntheticImage
 from skrt.registration import (
-        Registration, RegistrationEngine,
+        Elastix, NiftyReg, Registration, RegistrationEngine,
         add_engine, engines, get_default_pfiles, get_default_pfiles_dir,
-        get_engine_cls, read_parameters, set_elastix_dir, set_engine_dir)
+        get_engine_cls, get_jacobian_colormap, get_parameters, read_parameters,
+        set_elastix_dir, set_engine_dir)
 
 from test_structs import compare_rois
 
@@ -24,6 +28,20 @@ reg_dir = "tmp/reg"
 if os.path.exists(reg_dir):
     shutil.rmtree(reg_dir)
     
+# Directory for NiftyReg files.
+niftyreg_dir = Path("tmp/niftyreg")
+if niftyreg_dir.exists():
+    shutil.rmtree(str(niftyreg_dir))
+niftyreg_dir.mkdir()
+
+# Test affine matrix.
+affine = [
+        [11, 12, 13, 14],
+        [21, 22, 23, 24],
+        [31, 32, 33, 34],
+        [41, 42, 43, 44],
+        ]
+
 # Test images
 sim1 = SyntheticImage((10, 12, 8))
 sim1.add_cube(2, centre=(4, 4, 4))
@@ -345,6 +363,27 @@ def test_adjust_parameters():
     assert "(DefaultPixelValue 10)" in text
     os.remove(new_file)
 
+def test_get_parameters():
+    """Test parameter retrieval from an elastix parameter file."""
+    
+    pattern = "MI_Translation.txt"
+
+    params = {"DefaultPixelValue": 0}
+    parameters = get_parameters(pattern)
+    for key, value in params.items():
+        assert key in parameters
+        assert parameters[key] == value
+
+    params = {"DefaultPixelValue": 10}
+    parameters = get_parameters(pattern, params=params)
+    for key, value in params.items():
+        assert key in parameters
+        assert parameters[key] == value
+
+    with pytest.raises(IndexError) as error_info:
+        get_parameters(pattern, idx=5)
+    assert ("list index out of range" in str(error_info.value))
+
 @needs_elastix
 def test_shift_parameters():
     """Test shifting translation parameters by a given amount."""
@@ -584,12 +623,12 @@ def test_set_engine_dir():
         with pytest.raises(RuntimeError) as error_info:
             set_engine_dir(path="unknown", engine=engine)
         assert ("not implemented" in str(error_info.value)
-                or "executable(s) not found" in str(error_info.value))
+                or "not found" in str(error_info.value))
 
     # Check that exception is raised if engine name is unknown.
     with pytest.raises(RuntimeError) as error_info:
         set_engine_dir(path="unknown", engine="unknown")
-    assert "engine not known" in str(error_info.value)
+    assert "executable(s) not found" in str(error_info.value)
 
 
 def test_set_elastix_dir():
@@ -604,3 +643,306 @@ def test_set_elastix_dir():
                     or "executable(s) not found" in str(error_info.value))
         assert 1 == len(warning_info)
         assert "set_elastix_dir() deprecated" in warning_info[0].message.args[0]
+
+def not_implemented(engine, method):
+    """Test behaviour for method not implemented for a registration engine."""
+    # Create list of null parameter values to pass to method.
+    args = len(signature(getattr(engine, method)).parameters) * [None]
+
+    # Check that call to method raises an exception.
+    with pytest.raises(NotImplementedError) as error_info:
+        getattr(engine, method)(*args)
+    assert (f"not implemented for class {type(engine)}"
+            in str(error_info.value))
+
+def check_default_pfiles(engine, files=True):
+    """
+    Check availability of default parameter files for a registration engine.
+    """
+    pfiles = engine.get_default_pfiles(basename_only=True)
+    assert isinstance(pfiles, list)
+    if files:
+        assert pfiles
+        assert isinstance(pfiles[0], str)
+        pfiles = engine.get_default_pfiles(basename_only=False)
+        assert isinstance(pfiles, list)
+        assert pfiles
+        assert isinstance(pfiles[0], Path)
+    else:
+        assert not pfiles
+
+def ensure_engine_files(engine, exe, top_dir="tmp/engines"):
+    """
+    Ensure existence of dummy files used in testing registration engines.
+
+    **Parameters:**
+    engine: str
+        Name of registration engine.
+
+    exe: str
+        Name of executable that will be searched for when setting up
+        environment for running registration engine.
+
+    top_dir: str, default="tmp/engines"
+        Path to top-level directory of registration-engine files.
+    """
+    engine_dir = Path(fullpath(top_dir)) / engine
+    bin_dir = engine_dir / "bin"
+    bin_dir.mkdir(parents=True, exist_ok=True)
+    # Ensure existence of dummy files corresponding to masks and executables.
+    for filepath in [
+            engine_dir / "fixed_mask.nii", engine_dir / "moving_mask.nii",
+            bin_dir / exe, bin_dir / f"{exe}.exe"]:
+        if not filepath.exists():
+            with open(filepath, "w") as file:
+                pass
+    return engine_dir
+
+def test_registration_engine_define_translation():
+    """Test RegistrationEngine method define_translation()."""
+    not_implemented(RegistrationEngine(), "define_translation")
+
+def test_registration_engine_get_default_pfiles():
+    """Test RegistrationEngine method get_default_pfiles()."""
+    check_default_pfiles(RegistrationEngine(), False)
+
+def test_registration_engine_get_def_cmd():
+    """Test RegistrationEngine method get_def_cmd()."""
+    not_implemented(RegistrationEngine(), "get_def_cmd")
+
+def test_registration_engine_get_jac_cmd():
+    """Test RegistrationEngine method get_jac_cmd()."""
+    not_implemented(RegistrationEngine(), "get_jac_cmd")
+
+def test_registration_engine_get_registration_cmd():
+    """Test RegistrationEngine method get_registration_cmd()."""
+    not_implemented(RegistrationEngine(), "get_registration_cmd")
+
+def test_registration_engine_get_roi_params():
+    """Test RegistrationEngine method get_roi_params()."""
+    assert {} == RegistrationEngine().get_roi_params()
+
+def test_registration_engine_get_transform_cmd():
+    """Test RegistrationEngine method get_transforma_cmd()."""
+    not_implemented(RegistrationEngine(), "get_transform_cmd")
+
+def test_registration_engine_get_transform_log():
+    """Test RegistrationEngine method get_transform_log()."""
+    not_implemented(RegistrationEngine(), "get_transform_log")
+
+def test_registration_engine_set_exe_paths():
+    """Test RegistrationEngine method set_exe_paths()."""
+    not_implemented(RegistrationEngine(), "set_exe_paths")
+
+def test_elastix_define_translation():
+    """Test Elastix method define_translation()."""
+    engine = Elastix()
+    translation = (1, 5, -3)
+    params = engine.define_translation(translation)
+    assert isinstance(params, dict)
+    assert translation == params["TransformParameters"]
+    params = engine.define_translation(translation, sim1)
+    assert sim1.get_size() == params["Size"]
+
+def test_elastix_get_default_pfiles():
+    """Test Elastix method get_default_pfiles()."""
+    check_default_pfiles(Elastix())
+
+def test_elastix_get_def_cmd():
+    """Test Elastix method get_def_cmd()."""
+    args = ["fixed_path", "outdir", "tfile"]
+    def_cmd = Elastix().get_def_cmd(*args)
+    for arg in args[1:]:
+        assert arg in def_cmd
+
+def test_elastix_get_jac_cmd():
+    """Test Elastix method get_jac_cmd()."""
+    args = ["fixed_path", "outdir", "tfile"]
+    jac_cmd = Elastix().get_jac_cmd(*args)
+    for arg in args[1:]:
+        assert arg in jac_cmd
+
+def test_elastix_get_registration_cmd():
+    """Test Elastix method get_registration_cmd()."""
+    # Ensure existence of files checked for when creating registration command.
+    engine_dir = ensure_engine_files("elastix", "elastix")
+
+    # Define arguments, and create registration command.
+    args = ["fixed_path", "moving_path", str(engine_dir / "fixed_mask.nii"),
+            str(engine_dir / "moving_mask.nii"), "pfile", "outdir", "tfile"]
+    reg_cmd = Elastix().get_registration_cmd(*args)
+
+    # Check that expected arguments included in registration command.
+    for arg in args:
+        assert arg in reg_cmd
+
+def test_elastix_get_roi_params():
+    """Test Elastix method get_roi_params()."""
+    roi_params = Elastix().get_roi_params()
+    assert isinstance(roi_params, dict)
+    assert roi_params
+
+def test_elastix_get_transform_cmd():
+    """Test Elastix method get_transforma_cmd()."""
+
+    # Define arguments, and create transform command.
+    args = ["fixed_path", "moving_path", "outdir", "tfile", {"key": "value"}]
+    transform_cmd = Elastix().get_transform_cmd(*args)
+    print("transform_cmd", transform_cmd)
+
+    # Check that expected arguments included in transform command.
+    for arg in [args[1], args[2], f"{args[2]}/{args[3]}"]:
+        assert arg in transform_cmd
+
+def test_elastix_set_exe_paths():
+    """Test Elastix method set_exe_paths()."""
+    # Ensure existence of executable checked for when setting paths.
+    engine_dir = ensure_engine_files("elastix", "elastix")
+
+    # Create instance of registration engine.
+    engine = Elastix()
+
+    # Check that when the path to the executables directory is passed directly,
+    # the executables have suffix "exe".
+    exe_dir = engine.set_exe_paths(engine_dir / "bin")
+    assert exe_dir == engine_dir / "bin"
+    for exe in type(engine).exes:
+        assert getattr(engine, exe)  == f"{exe}.exe"
+
+    # Check that when the path to the directory above the executables path
+    # is passed, the executables don't have suffix "exe".
+    exe_dir = engine.set_exe_paths(engine_dir)
+    assert exe_dir == engine_dir / "bin"
+    for exe in type(engine).exes:
+        assert getattr(engine, exe)  == exe
+
+def test_niftyreg_define_translation():
+    """Test NiftyReg method define_translation()."""
+    engine = NiftyReg()
+    translation = (1, 5, -3)
+    signs = (-1, -1, 1)
+    params = engine.define_translation(translation)
+    assert isinstance(params, list)
+    for idx, dxyz in enumerate(translation):
+        assert str(signs[idx]* dxyz) == params[idx].split()[-1]
+
+def test_nifyreg_get_default_pfiles():
+    """Test NiftyReg method get_default_pfiles()."""
+    check_default_pfiles(NiftyReg())
+
+def test_niftyreg_get_def_cmd():
+    """Test NiftyReg method get_def_cmd()."""
+    args = ["fixed_path", "outdir", "tfile"]
+    def_cmd = NiftyReg().get_def_cmd(*args)
+    for arg in args:
+        assert any(arg in item for item in def_cmd)
+
+def test_niftyreg_get_jac_cmd():
+    """Test NiftyReg method get_jac_cmd()."""
+    args = ["fixed_path", "outdir", "tfile.nii"]
+    jac_cmd = NiftyReg().get_jac_cmd(*args)
+    for arg in args:
+        assert any(arg in item for item in jac_cmd)
+
+def test_niftyreg_get_registration_cmd():
+    """Test NiftyReg method get_registration_cmd()."""
+    # Ensure existence of files checked for when creating registration command.
+    engine_dir = ensure_engine_files("niftyreg", "reg_f3d")
+
+    # Define arguments, and create registration command.
+    pfiles = [str(get_default_pfiles("Rigid.txt", "niftyreg")[0]),
+              str(get_default_pfiles("*BSpline15*", "niftyreg")[0])]
+    for pfile in pfiles:
+        args = ["fixed_path", "moving_path", str(engine_dir / "fixed_mask.nii"),
+                str(engine_dir / "moving_mask.nii"), pfile, "outdir", "tfile"]
+        reg_cmd = NiftyReg().get_registration_cmd(*args)
+
+        # Check that expectedarguments included in registration command.
+        for arg in args:
+            if pfile != arg:
+                assert any(arg in item for item in reg_cmd)
+        params = read_parameters(pfile)
+        assert params["exe"] in reg_cmd
+
+def test_niftyreg_get_roi_params():
+    """Test NiftyReg method get_roi_params()."""
+    roi_params = NiftyReg().get_roi_params()
+    assert isinstance(roi_params, dict)
+    assert roi_params
+
+def test_niftyreg_get_transform_cmd():
+    """Test NiftyReg method get_transform_cmd()."""
+
+    # Define arguments, and create transform command.
+    args = ["fixed_path", "moving_path", "outdir", "tfile"]
+    transform_cmd = NiftyReg().get_transform_cmd(*args)
+
+    # Check that expected arguments included in transform command.
+    for arg in args:
+        assert any(arg in item for item in transform_cmd)
+
+def test_niftyreg_set_exe_paths():
+    """Test NiftyReg method set_exe_paths()."""
+    # Ensure existence of executable checked for when setting paths.
+    engine_dir = ensure_engine_files("niftyreg", "reg_f3d")
+
+    # Create instance of registration engine.
+    engine = NiftyReg()
+
+    # Check that when the path to the executables directory is passed directly,
+    # the executables have suffix "exe".
+    exe_dir = engine.set_exe_paths(engine_dir / "bin")
+    assert exe_dir == engine_dir / "bin"
+    for exe in type(engine).exes:
+        assert getattr(engine, exe)  == f"{exe}.exe"
+
+    # Check that when the path to the directory above the executables path
+    # is passed, the executables don't have suffix "exe".
+    exe_dir = engine.set_exe_paths(engine_dir)
+    assert exe_dir == engine_dir / "bin"
+    for exe in type(engine).exes:
+        assert getattr(engine, exe)  == exe
+
+def test_get_jacobian_colormap():
+    cmap = get_jacobian_colormap()
+    assert isinstance(cmap, matplotlib.colors.LinearSegmentedColormap)
+    assert "jacobian" == cmap.name
+
+def test_niftyreg_write_and_read_affine():
+    """Test writing and reading of affine matrix."""
+    niftyreg = NiftyReg()
+
+    # Write affine matrix to file.
+    transform_path = niftyreg_dir / "transform.txt"
+    transform_path.unlink(missing_ok=True)
+    assert not transform_path.exists()
+    niftyreg.write_affine(affine, transform_path)
+    assert transform_path.exists()
+
+    # Read affine matrix from file.
+    affine_from_file = niftyreg.read_affine(transform_path)
+    assert affine == affine_from_file
+
+def test_niftyreg_shift_translation_parameters():
+    """Test shifting of translation parameters for NiftyReg affine transform."""
+    niftyreg = NiftyReg()
+
+    # Define offsets, and signs to match elastix convention.
+    dxdydz = (94, 2, 5)
+    signs = (-1, -1, 1)
+
+    # Write initial affine matrix to file.
+    transform_path = niftyreg_dir / "transform.txt"
+    niftyreg.write_affine(affine, transform_path)
+    assert transform_path.exists()
+
+    # Apply shifts, and check that result is as expected.
+    shifted_path = niftyreg_dir / "shifted.txt"
+    niftyreg.shift_translation_parameters(transform_path, *dxdydz, shifted_path)
+    shifted = niftyreg.read_affine(shifted_path)
+    for irow, row in enumerate(shifted):
+        for icol, val in enumerate(row):
+            if irow <= 2 and icol == 3:
+                assert val == affine[irow][icol] + signs[irow] * dxdydz[irow]
+            else:
+                assert val == affine[irow][icol]
