@@ -2,11 +2,17 @@
 
 import pytest
 
+from skrt import Image, StructureSet
+from skrt.core import Defaults
 from skrt.segmentation import (
+        ensure_image,
+        ensure_structure_set,
         get_contour_propagation_strategies,
         get_fixed_and_moving,
         get_option,
         get_segmentation_steps,
+        get_steps,
+        get_structure_set_index,
         MultiAtlasSegmentation,
         SingleAtlasSegmentation,
         SasTuner,
@@ -67,6 +73,30 @@ def test_sas_tuner_instantiation():
     assert tuner.kwargs == {}
     assert tuner.df is None
 
+def test_ensure_image():
+    """Test that ensure_image() returns Image or None."""
+    assert ensure_image(None) is None
+    assert isinstance(ensure_image(Image()), Image)
+
+def test_ensure_structure_set():
+    """Test that ensure_structure_set() returns StructureSet or None."""
+    assert ensure_structure_set(None) is None
+    assert isinstance(ensure_structure_set(StructureSet()), StructureSet)
+
+def test_get_contour_propagation_strategies():
+    """
+    Test that get_contour_propagation_strategies() returns list of strings.
+    """
+    local_engines = {"niftyreg": 1, "elastix": 2}
+    for engine, n_strategy in local_engines.items():
+        for engine_arg in [engine, None]:
+            Defaults().registration_engine = engine
+            strategies = get_contour_propagation_strategies(engine_arg)
+            assert isinstance(strategies, list)
+            assert len(strategies) == n_strategy
+            for strategy in strategies:
+                assert isinstance(strategy, str)
+
 def test_get_fixed_and_moving():
     """Test image assignment as fixed or moving, depending on strategy"""
     # There's no check of object type, so test with strings.
@@ -105,3 +135,22 @@ def test_get_option():
     for invalid, allowed in zip(invalid_opts, allowed_opts):
         assert get_option(opt=invalid, fallback_opt=allowed,
                 allowed_opts=allowed_opts) == allowed
+
+def test_get_segmentation_steps():
+    """Test retrieval of segmentation steps."""
+    seg_steps = get_segmentation_steps()
+    assert isinstance(seg_steps, list)
+    assert len(seg_steps) > 0
+
+def test_get_steps():
+    """Test retrieval of steps to be run."""
+    seg_steps = get_segmentation_steps()
+    assert get_steps(None) == seg_steps
+    for idx, name in enumerate(seg_steps):
+        n_step = idx + 1
+
+        for step in [idx, name, [idx], [name]]:
+            steps = get_steps(step)
+            assert len(steps) == n_step
+            for idx2 in range(n_step):
+                assert seg_steps[idx2] in steps
