@@ -14,12 +14,14 @@ import matplotlib.colors
 from shapely.geometry import Polygon
 from shapely.validation import explain_validity
 
+from skimage.morphology import ball
+
 from skrt.core import fullpath, Defaults
 from skrt.simulation import SyntheticImage
 from skrt.structures import contour_to_polygon, polygon_to_contour, \
         StructureSet, ROI, interpolate_points_single_contour, \
         get_comparison_metrics, get_slice_positions, expand_slice_stats, \
-        get_metric_method
+        get_metric_method, get_structuring_element
 
 
 # Make temporary test dir
@@ -112,6 +114,24 @@ def test_structure_set_from_rois():
     for key in sdict1:
         assert key in sdict2
 
+def test_has_rois():
+    """Test determination of whether structure set contains specified ROIs."""
+    # Specifications of ROIs contained in structure set.
+    has_rois = ["CUBE", ["C*", "S*"], 
+                {"cuboid": "cube", "non_cuboid": ["cylinder", "sphere"]}]
+    # Specifications of ROIs not contained in structure set.
+    not_has_rois = ["cylinder", ["cube", "cylinder"],
+                    {"cuboid": "cube", "non_cuboid": ["cylinder"]}]
+    # ROI specified via key.
+    roi_as_key = {"cube": "large_cube"}
+
+    for rois in has_rois:
+        assert structure_set.has_rois(rois)
+    for rois in not_has_rois:
+        assert not structure_set.has_rois(rois)
+    for include_keys in [True, False]:
+        assert structure_set.has_rois(roi_as_key, include_keys) is include_keys
+
 def test_structure_set_addition():
     """Test addition of structure sets."""
 
@@ -1091,24 +1111,62 @@ def test_roi_split_in_two():
 def test_combine_rois():
     '''Test combining of ROIs.'''
 
-    # Define non-composite ROIs.
-    sim = SyntheticImage((100, 100, 100))
-    sim.add_cube(side_length=40, name="cube", centre=(30, 30, 50), intensity=1)
-    sim.add_sphere(radius=20, name="sphere", centre=(70, 70, 50), intensity=10)
-    cube = sim.get_roi('cube')
-    sphere = sim.get_roi('sphere')
-    ss = sim.get_structure_set()
-    
-    # Use different methods for defining composite ROI.
-    for method in ["auto", "mask", "contour"]:
-        cube_and_sphere = ss.combine_rois(method=method)
+    # Consider two situations:
+    # - disjoint sphere and cube (different centres);
+    # - sphere fully contained in cube (same centre).
+    cube_centre = (30, 30, 50)
+    for sphere_centre in [(70, 70, 50), cube_centre]:
+        # Define non-composite ROIs.
+        sim = SyntheticImage((100, 100, 100))
+        sim.add_cube(side_length=40, name="cube", centre=cube_centre,
+                     intensity=1)
+        sim.add_sphere(radius=20, name="sphere", centre=sphere_centre,
+                       intensity=10)
+        cube = sim.get_roi('cube')
+        sphere = sim.get_roi('sphere')
+        ss = sim.get_structure_set()
+        cube_and_sphere_name = "+".join(ss.get_roi_names())
+        same_centre = np.all(cube.get_centre() == sphere.get_centre())
 
-        assert (cube_and_sphere.get_volume() == pytest.approx(
-            cube.get_volume() + sphere.get_volume(), rel=0.001))
+        # Use different methods for defining composite ROI.
+        for method in ["auto", "mask", "contour"]:
+            # Consider combination through union and through intersection.
+            for intersection in [False, True]:
+                cube_and_sphere = ss.combine_rois(
+                        method=method, intersection=intersection)
 
-        if method != "contour":
-            assert np.all(cube_and_sphere.get_mask() ==
-                    cube.get_mask() + sphere.get_mask())
+                # Check name of composite ROI.
+                prefix = "intersection:" if intersection else ""
+                assert cube_and_sphere.get_name() == (
+                        prefix + cube_and_sphere_name)
+
+                # Check volume of composite ROI.
+                if intersection:
+                    cube_and_sphere_volume = (
+                            0 if not same_centre else pytest.approx(
+                                sphere.get_volume(), rel=0.001))
+                else:
+                    cube_and_sphere_volume = (
+                            pytest.approx(cube.get_volume()
+                                          + sphere.get_volume(), rel=0.001)
+                            if not same_centre else
+                            pytest.approx(cube.get_volume(), rel=0.001))
+                assert cube_and_sphere.get_volume() == cube_and_sphere_volume
+
+                # Check mask of composite ROI.
+                if method != "contour":
+                    if intersection:
+                        if same_centre:
+                            assert (np.all(cube_and_sphere.get_mask()
+                                           == sphere.get_mask()))
+                    else:
+                        if same_centre:
+                            assert np.all(cube_and_sphere.get_mask()
+                                          == cube.get_mask())
+                        else:
+                            assert np.all(cube_and_sphere.get_mask()
+                                          == cube.get_mask()
+                                          + sphere.get_mask())
 
 def test_mask_image():
     '''Test retrieval of image objects repreenting ROI masks.'''
@@ -1681,3 +1739,110 @@ def test_missing():
     # Reduced-size image doesn't contain ROIs.
     sim2 = SyntheticImage([dxyz // 2 for dxyz in sim.get_n_voxels()])
     assert structure_set.missing(roi_names, in_image=sim2) == roi_names
+
+def test_get_intensities_3d():
+    """Test calculation of 3D array with intensity values inside ROI."""
+    sim = SyntheticImage((10, 10, 10), origin=(0.5, 0.5, 0.5), intensity=0)
+    sim.add_cuboid((4, 2, 6), name="cube", intensity=100)
+    roi = sim.get_roi("cube")
+    for image in [None, sim]:
+        intensities = roi.get_intensities_3d(image)
+        sim_data = sim.get_data()
+        nan = np.isnan(intensities)
+        assert np.all(intensities[~nan] == sim_data[~nan])
+        assert np.all(sim_data[nan] == 0)
+
+def test_get_intensities():
+    """Test calculation of 1D array of intensity values inside ROI."""
+    sim = SyntheticImage((10, 10, 10), origin=(0.5, 0.5, 0.5), intensity=0)
+    sim.add_cuboid((4, 2, 6), name="cube", intensity=100)
+    roi = sim.get_roi("cube")
+    for image in [None, sim]:
+        intensities = roi.get_intensities(image)
+        assert len(intensities) == roi.get_volume()
+        assert np.all(intensities == 100)
+
+def test_get_structuring_element():
+    """Test generation of structuring element."""
+
+    # Check that element generated for different radii 
+    # matches result of skimage.morphology.ball,
+    # taking into account different voxel size.
+    for radius in range(1, 4):
+        assert np.all(get_structuring_element(radius) == ball(radius))
+        assert np.all(get_structuring_element(2 * radius, (2, 2, 2))
+                      == ball(radius))
+
+    # Check that element has correct shape
+    # when voxel size is different in each dimension.
+    radius = 4
+    voxel_size = (2, 4, 1)
+    element = get_structuring_element(radius, voxel_size)
+    assert element.shape == tuple([1 + 2 * radius / voxel_size[idx]
+                                   for idx in [1, 0, 2]])
+
+def test_resized():
+    """Test increasing and decreasing ROIs by different amounts."""
+    for sign, symbol in [(1, "+"), (-1, "-")]:
+        for margin in [1, 2, 4, 8]:
+            signed_margin = sign * margin
+            for roi in [sphere, cube]:
+                resized_roi = roi.resized(margin=(signed_margin))
+                assert all([(roi.get_bbox_centre_and_widths()[1][idx]
+                             + 2 * signed_margin)
+                             == resized_roi.get_bbox_centre_and_widths()[1][idx]
+                             for idx in range(3)])
+                assert resized_roi.name == f"{roi.name}{symbol}{margin}"
+    del resized_roi
+
+def test_resize_contours():
+    """Test resizing ROI contours by different amounts."""
+
+    for dxy in [-4.5, -1, 0, 1, 4.5]:
+        for roi1 in [cube, sphere]:
+            roi2 = roi1.clone()
+            roi2.resize_contours(dxy=dxy)
+
+            width1 = roi1.get_bbox_centre_and_widths(method="contour")[1]
+            width2 = roi2.get_bbox_centre_and_widths(method="contour")[1]
+            for idx in [0, 1]:
+                assert width1[idx] + 2 * dxy == width2[idx]
+            assert width1[2] == width2[2]
+
+            del roi2
+
+def test_roi_algebra():
+    """Test ROI addition, subtraction, and multiplication."""
+
+    # Test algebraic operations with cube and sphere.
+    xyz1 = (30, 30, 50)
+    xyz2 = (70, 70, 50)
+    for cube_centre in [xyz1]:
+        for sphere_centre in [xyz1, xyz2]:
+            sim = SyntheticImage((100, 100, 100))
+            sim.add_cube(side_length=40, name="cube", centre=cube_centre,
+                         intensity=1)
+            sim.add_sphere(radius=20, name="sphere", centre=sphere_centre,
+                           intensity=10)
+            cube = sim.get_roi('cube')
+            sphere = sim.get_roi('sphere')
+
+            if cube_centre == sphere_centre:
+                # Check results when cube and sphere have same centre.
+                assert (cube + sphere).get_volume() == cube.get_volume()
+                assert (cube * sphere).get_volume() == sphere.get_volume()
+                assert ((cube - sphere).get_volume()
+                        == cube.get_volume() - sphere.get_volume())
+                assert np.all((cube + sphere).get_mask() == cube.get_mask())
+                assert np.all((cube * sphere).get_mask() == sphere.get_mask())
+                assert np.all((cube - sphere).get_mask()
+                              == cube.get_mask() & ~sphere.get_mask())
+            else:
+                # Check results when cube and sphere are disjoint.
+                assert ((cube + sphere).get_volume()
+                        == cube.get_volume() + sphere.get_volume())
+                assert (cube * sphere).get_volume() == 0
+                assert (cube - sphere).get_volume() == cube.get_volume()
+                assert np.all((cube + sphere).get_mask()
+                              == cube.get_mask() | sphere.get_mask())
+                assert np.all((cube - sphere).get_mask() == cube.get_mask())
